
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "ez-bitset.h"

#include <assert.h>

/* copy internal decl from ez-bitset.c */
#define L0B 9
#define L1B 9
#define L2B 14

#define L0 (1<<L0B)
#define L1 (1<<L1B)
#define L2 ((1<<L2B)/32)

#define L0S (L1B+L2B)
#define L1S (L2B)
#define L2S (5)

#define L0M (L0-1)
#define L1M (L1-1)
#define L2M (L2-1)

#define L3M 31

struct level2 {
	uint32_t bits[L2];
};

struct level1 {
	struct level2 *level2[L1];
};

struct ez_bitset {
	struct level1 *level1[L0];
	uint32_t count;
	// default?
};

/* a bunch of shit tests */

void info(ez_bitset *bs) {
	printf("L0: count=%d shift=%d mask=%08x\n", L0, L0S, L0M);
	printf("L1: count=%d shift=%d mask=%08x\n", L1, L1S, L1M);
	printf("L2: count=%d shift=%d mask=%08x\n", L2, L2S, L2M);
	
	for (int id0 = 0;id0<L0;id0++) {
		struct level1 *l1 = bs->level1[id0];

		if (l1) {
			for (int id1 = 0; id1 < L1; id1++) {
				struct level2 *l2 = l1->level2[id1];

				if (l2) {
					for (int id2=0;id2<L2;id2++) {
						if (l2->bits[id2])
							printf(" %4d.%4d.%4d %08x\n", id0, id1, id2, l2->bits[id2]);
					}
				}
			}
		}
	}
}

int main(int argc, char **argv) {
	ez_bitset *bs = ez_bitset_new();
	int total = 0;
	for (int i=1500;i<2060;i++) {
		ez_bitset_set(bs, i, 1);
		total ++;
	}

	assert(ez_bitset_card(bs) == total);
	
	ez_bitset_scan scan;
	uint32_t bit, check;


	info(bs);

	check = 1500;
	for (bit = ez_bitset_scan_init(bs, &scan); bit != ~0; bit = ez_bitset_scan_next(&scan)) {
		//printf(" %d\n", bit);
		assert(check == bit);
		check++;
	}

	//printf("card = %d\n", ez_bitset_card(bs));
	check = 1500;
	for (bit = ez_bitset_scan_init(bs, &scan); bit != ~0; bit = ez_bitset_scan_remove(&scan)) {
		//printf(" remove %d\n", bit);
		assert(check == bit);
		check++;
	}
	
	assert(ez_bitset_card(bs) == 0);
	//printf("card = %d\n", ez_bitset_card(bs));

	check = 0;
	for (bit = ez_bitset_scan_init(bs, &scan); bit != ~0; bit = ez_bitset_scan_next(&scan)) {
		check++;
	}
	assert(check == 0);

	ez_bitset_free(bs);

	bs = ez_bitset_new();
	ez_bitset_clear(bs);
	total = 0;
	for (int i=65536;i<65536+(1<<20);i+=7) {
		ez_bitset_set(bs, i, i);
		total += 1;
	}
	assert(ez_bitset_card(bs) == total);

	//info(bs);
	for (int i=65536;i<65536+(1<<20);i++) {
		int v = (i-65536) % 7 == 0;
		
		assert(v == ez_bitset_get(bs, i));
	}

	ez_bitset_clear(bs);
	for (int i=65536-65;i<65536+(1<<20)+65;i++) {
		assert(0 == ez_bitset_get(bs, i));
	}

	ez_bitset_free(bs);

	return 0;
}

